# JwPlot.rb @JWC / JWW Oό`t@CXNvg  Rev. 1.30

# CLASS: Oό`t@C̃NX
class JwPlotFile
	attr	:font,				true
	attr	:lines,				true
	attr	:points,			true
	attr	:solids,			true
	attr	:scale,				true
	attr	:path,				true
	attr	:pen,					true
	attr	:source_file,	true
	attr	:ignore_attr,	true

	def read()
		@font = JwFont.new
		@lines = Array.new
		@points = Array.new
		@strings = Array.new
		@solids = Array.new
		@positions = Hash.new
		@scale = JwScale.new
		@pen = JwPen.new
		group = nil
		@source_file = ''

		if @pwd != nil
			filename = @pwd + ARGV[ 0 ]
		elsif @path != nil
			filename = @path + ARGV[ 0 ]
		else
			filename = ARGV[ 0 ]
		end

		jw = open( filename, 'r' )
		begin
			while jw.gets
				chomp!
				words = split
				case $_
					when /^hp([0-9]+)/
						@positions[ $1.to_i ] = Point.new( words[1].to_f, words[2].to_f )
	
					when /^cn/
						@font.id = $_[2 .. -1].to_i
						@default_font = @font.id if @default_font == nil
	
					when /^(hcw|hch|hcd)/
						@font.set $_
	
					when /^ly(.*)/
						if @layer == nil
							@layer = $1.to_i
						end
						current_layer = $1
	
					when /^hs/
						@scale.set $_
		
					when /^lg(.*)/
						if group == nil
							@scale.set $_
							group = $1
						end
	
					when /^lc([1-9])$/
						if @lineColor == nil
							@lineColor = $1.strip
							@pen.set( $_ )
						end
						pen = $1.strip
						current_color = pen

					when /^lt([1-9])$/
						if @lineStyle == nil
							@lineStyle = $1.strip
							@pen.set( $_ )
						end
						pen = $1.strip
						current_style = pen

					when /^ *\-*[0-9]/
						if words.length == 4

							if @ignore_attr == true
								@lines.push Line.new(
												words[ 0 ].to_f, words[ 1 ].to_f,
												words[ 2 ].to_f, words[ 3 ].to_f
								)
							else
								@lines.push JwLine.new(
												words[ 0 ].to_f, words[ 1 ].to_f,
												words[ 2 ].to_f, words[ 3 ].to_f
								)
								@lines.last.pen.color = current_color.to_i
								@lines.last.pen.type = current_style.to_i
								@lines.last.layer = current_layer
							end

							@lines.last.sort
						end

					when /^lc10 (.*)/
						solid_color = $1.to_i
						pen = "10"

					when /^pt/
						if words.length == 3
							@points.push Point.new( words[1].to_f, words[2].to_f )
						end

					when /^ch/
						@strings.push JwString.new( $_ )

					when /^sl/
						@solids << SolidPolygon.new( $_ )
						@solids.last.pen = pen
						if pen == "10"
							@solids.last.color = solid_color
						end

					when /^file=(.*)/
						@source_file = $1

				end
			end
		ensure
			jw.close
		end
	end

	def checkDefaultLayer( layer )
    if layer != "Z"
    	new_layer = layer
    else
    	new_layer = @layer
    end

		new_layer
	end

	def checkDefaultColor( color )
    if color != "0"
      new_color = color
    else
      new_color = @lineColor
    end
 
    new_color
	end

	def checkDefaultStyle( style )
    if style != "0"
      new_style = style
    else
    	if @lineStyle != nil
      	new_style = @lineStyle
      else
      	new_style = "1"
      end
    end
 
    new_style
	end
end

# CLASS: _
class Point
	attr	:x, true
	attr	:y, true
	attr	:sortHorizontal,	true
	attr	:sortAscent,			true

	extend Comparable

	def initialize( *args )
		if args.length > 0
			@x = args[0]
			@y = args[1]
		else
			@x = 0.0
			@y = 0.0
		end
		@sortHorizontal = true
		@sortAscent = true
	end

	def draw( *out )
		if out[ 0 ].is_a? File
			out[ 0 ].printf "pt %.10f %.11f\n", @x, @y
		else
			printf "pt %.10f %.11f\n", @x, @y
		end
	end

	# METHOD: 
	def distance( other )
		x = @x - other.x
		y = @y - other.y
		Math.sqrt( x * x + y * y )
	end

	# Z
	def +( other )
		new_pt = self.clone
		new_pt.x += other.x
		new_pt.y += other.y
		new_pt
	end

	# Z
	def -( other )
		new_pt = self.clone
		new_pt.x -= other.x
		new_pt.y -= other.y
		new_pt
	end

	# rZq: 
	def ===( other )
		return ( @x == other.x  and  @y == other.y )
	end

	# rZq: 召risortpj
	def <=>( other )
		if @sortAscent
			ascent = 1
		else
			ascent = -1
		end

		if @sortHorizontal
			if @x > other.x
				result = ascent
			elsif @x < other.x
				result = - ascent
			else
				if @y > other.y
					result = -1
				elsif @y < other.y
					result = 1
				else
					result = 0
				end
			end
		else
			if @y > other.y
				result = ascent
			elsif @y < other.y
				result = - ascent
			else
				if @x > other.x
					result = 1
				elsif @x < other.x
					result = -1
				else
					result = 0
				end
			end
		end

		result
	end

	# METHOD: ߐړ_̔
	def touched?( other, tolerance )
		if other.kind_of? Point
			((other.x - @x).abs < tolerance) and ((other.y - @y).abs < tolerance)
		else
			false
		end
	end
end

# CLASS: _i_w,[~j
class JwPoint < Point
	attr	:type,		true
	attr	:color,		true
	attr	:radius,	true
	attr	:scale,		true

	def initialize( *args )
		super( *args )
		@pen = JwPen.new
		@circle = Ellipse.new
		@radius = 0.0
		@circle.center = self
	end

	def draw( *out )
		if @type != nil
			if @type == 'C'
				if ( @color != nil )
					@pen.color = @color
				end
				@circle.radius = @radius
				@circle.radius = @scale.value if @scale != nil
				@pen.draw( *out )
				@circle.draw( *out )
			else
				if out[ 0 ].is_a? File
					out[ 0 ].printf( "pn%d\n", @type )
				else
					printf( "pn%d\n", @type )
				end
				super( *out )
			end
		else
			super( *out )
		end
	end
end

# CLASS: 
class Line
	def initialize( *args )
		if args.length > 0
			if args[ 0 ].kind_of? Numeric
				@from = Point.new( args[ 0 ], args[ 1 ] )
				@to = Point.new( args[ 2 ], args[ 3 ] )
			elsif args[ 0 ].kind_of? Point
				@from = args[ 0 ]
				@to = args[ 1 ]
			end
		end
	end
	attr("from", true)
	attr("to", true)
  
	def length
		x = @from.x - @to.x
		y = @from.y - @to.y
		Math.sqrt( x * x + y * y )
	end

	def sort
		if from.y > to.y
			 ptr = @from
			 @from = @to
			 @to = ptr
		end
		if from.x > to.x
			 ptr = @from
			 @from = @to
			 @to = ptr
		end
	end

	def left=( x )
		old_left = [ @from.x, @to.x ].min
		dx = x - old_left
		@from.x += dx
		@to.x += dx
	end

	def center
		return Point.new( ( @from.x + @to.x ) * 0.5, ( @from.y + @to.y ) * 0.5 )
	end

	def width
		(@to.x - @from.x)
	end

	def	height
		(@to.y - @from.y)
	end

	def writeSize( margin, font, format, *out )
		ptr = self.center

		size_str = JwString.new
		size_str.font = font
		format.number = length / 1000
		size_str.text = format.to_s.strip
		size_str.direction.x = width.abs
		if width >= 0.0
			size_str.direction.y = height
		else
			size_str.direction.y = - height
		end

		ptr.x -= margin * size_str.direction.y / length
		ptr.y += margin * size_str.direction.x / length
		size_str.bottom_center = ptr
		size_str.draw *out
	end

	def draw( *out )
		if out[ 0 ].is_a? File
			out[ 0 ].printf " %.10f %.11f %.10f %.11f\n", @from.x, @from.y, @to.x, @to.y
		else
			printf " %.10f %.11f %.10f %.11f\n", @from.x, @from.y, @to.x, @to.y
		end
	end

	# rZq: s
	def <=>( other )
		sort
		other.sort
		if @from.x > other.from.x
			result = 1
		elsif @from.x < other.from.x
			result = -1
		elsif @from.y > other.from.y
			result = 1
		elseif @from.y < other.from.y
			result = -1
		else
			result = 0
		end
		result
	end

	# rZq: 
	def ==( other )
		sort
		other.sort
		if @from === other.from && @to === other.to
			result = true
		else
			result = false
		end
		result
	end

	# 
	def intersect?( other )
		intersect = ((@from.x - @to.x) * (other.from.y - @from.y) + 
									(@from.y - @to.y) * (@from.x - other.from.x)) * 
								((@from.x - @to.x) * (other.to.y - @from.y) + 
									(@from.y - @to.y) * (@from.x - other.to.x))
		( intersect <= 0.0 ? true : false )
	end

	# ߐڔ
	def touched?( other, tolerance )
		if other.kind_of? Line
			if ( @from.touched?(other.from, tolerance) or
						@from.touched?(other.to, tolerance) or
						@to.touched?(other.from, tolerance) or
						@to.touched?(other.to, tolerance) )
				true
			else
				false
			end
		elsif other.kind_of? Point
			if ( @from.touched?(other, tolerance) or
						@to.touched?(other, tolerance) )
				true
			else
				false
			end
		end
	end
end

# CLASS:  + Pen
class JwLine < Line
	attr	:pen,	true
	attr	:layer, true

	def initialize( *args )
		@pen = JwPen.new
		super( *args )
	end

	def draw( *out )
		if @layer != nil
			if out[ 0 ].is_a? File
				out[ 0 ].printf "ly%s\n", @layer
			else
				printf "ly%s\n", @layer
			end
		end

		@pen.draw *out
		super *out
	end
end

# ~̃NX
class Ellipse
	attr	:center,			true
	attr	:radius,			true
	attr	:aspect,			true
	attr	:axis,				true

	def initialize
		@center = Point.new(0,0)
		@radius = 0.0
		@start_angle = 0.0
		@end_angle = 360.0
		@aspect = 1.0
		@axis = 0.0
	end

	def draw( *out )
		if out[ 0 ].is_a? File
			out[ 0 ].printf "ci %.11f %.11f %.11f %.11f %.11f %.11f %.11f\n", @center.x, @center.y, @radius, @start_angle, @end_angle, @aspect, @axis
		else
			printf "ci %.11f %.11f %.11f %.11f %.11f %.11f %.11f\n", @center.x, @center.y, @radius, @start_angle, @end_angle, @aspect, @axis
		end
	end
end

# ~ʂ̃NX
class Arc < Ellipse
	attr	:start_angle,	true
	attr	:end_angle,		true
end

# \bh}`
class SolidPolygon
	attr	:color,	true
	attr	:pen,		true
	attr	:peaks,	true

	def initialize( *args )
		@peaks = Array.new
		@pen = "0"
		@color = 0
		if args[0].kind_of? String
			set args[0]
		end
	end

	def set( s )
		if s =~ /^sl/
			xy = s.split
			i = 1
			while i < xy.length
				@peaks << Point.new( xy[i].to_f, xy[i+1].to_f )
				i += 2
			end
		end
	end
end

# ̃NX
class JwString
	include Math
	attr	:position,	true
	attr	:direction, true
	attr	:text,			true
	attr	:font,			true
	attr	:font_id,		true
	attr	:scale,			true

	def initialize( *args )
		@position = Point.new(0,0)
		@direction = Point.new(1,0)
		@text = ""
		@scale = 1.0

		if args[0].kind_of? String
			set args[0]
		end
	end

	def set( s )
		if s =~ /ch (.+) (.+) (.+) (.+) \"(.*)/
			@position.x = $1.to_f
			@position.y = $2.to_f
			@direction.x = $3.to_f
			@direction.y = $4.to_f
			@text = $5
		else
			@text = s
		end
	end

	def draw( *out )
		@font = JwFont.new() if @font == nil
		if out[ 0 ].is_a? File
			if @font_id != nil
				if @font.id != @font_id
					@font.id = @font_id
					@font.draw( out[0] )
				end
			else
				@font.draw( out[0] )
			end
			out[ 0 ].printf(
								"ch %.11f %.11f %f %f \"%s\n",
								@position.x, @position.y,
								@direction.x, @direction.y,
								@text
			)
		else
			if @font_id != nil
				if @font.id != @font_id
					@font.id = @font_id
					@font.draw
				end
			else
				@font.draw
			end
			printf(
					"ch %.11f %.11f %f %f \"%s\n",
					@position.x, @position.y,
					@direction.x, @direction.y,
					@text
			)
		end
	end

	def width
		id = @font.id
		@font.id = @font_id	if @font_id != nil
		l = @font.stringSize( @text ) * 0.5 * @scale
		@font.id = id

		l
	end

	def charWidth
		id = @font.id
		@font.id = @font_id	if @font_id != nil
		l = (@font.width + @font.gap) * @scale
		@font.id = id

		l
	end

	def center=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		end

		id = @font.id
		@font.id = @font_id	if @font_id != nil
		l = @font.stringSize( @text ) * 0.5 * @scale
		h = @font.height * 0.5 * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x - l * @direction.x / ll + h * @direction.y / ll
		@position.y = y - l * @direction.y / ll - h * @direction.x / ll

		@font.id = id
	end

	def top_left=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		end

		id = @font.id
		@font.id = @font_id	if @font_id != nil
		h = @font.height * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x + h * @direction.y / ll
		@position.y = y - h * @direction.x / ll

		@font.id = id
	end

	def top_center=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		elsif pos[0].kind_of? Array
			if pos[0][0].kind_of? Numeric
				x = pos[0][0]
				y = pos[0][1]
			end
		end

		id = @font.id
		@font.id = @font_id	if @font_id != nil
		l = @font.stringSize( @text ) * 0.5 * @scale
		h = @font.height * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x - l * @direction.x / ll + h * @direction.y / ll
		@position.y = y - l * @direction.y / ll - h * @direction.x / ll

		@font.id = id
	end

	def top_right=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		elsif pos[0].kind_of? Array
			if pos[0][0].kind_of? Numeric
				x = pos[0][0]
				y = pos[0][1]
			end
		end

		id = @font.id
		@font.id = @font_id	if @font_id != nil
		l = @font.stringSize( @text ) * @scale
		h = @font.height * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x - l * @direction.x / ll + h * @direction.y / ll
		@position.y = y - l * @direction.y / ll - h * @direction.x / ll

		@font.id = id
	end

	def bottom_left=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		elsif pos[0].kind_of? Array
			if pos[0][0].kind_of? Numeric
				x = pos[0][0]
				y = pos[0][1]
			end
		end

		@position.x = x
		@position.y = y
	end

	def bottom_center=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		elsif pos[0].kind_of? Array
			if pos[0][0].kind_of? Numeric
				x = pos[0][0]
				y = pos[0][1]
			end
		end

		id = @font.id
		@font.id = @font_id	if @font_id != nil
		l = @font.stringSize( @text ) * 0.5 * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x - l * @direction.x / ll
		@position.y = y - l * @direction.y / ll		if ll != 0.0

		@font.id = id
	end

	def center_left=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		elsif pos[0].kind_of? Array
			if pos[0][0].kind_of? Numeric
				x = pos[0][0]
				y = pos[0][1]
			end
		end

		id = @font.id
		@font.id =  @font_id	if @font_id != nil
		h = @font.height * 0.5 * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x + h * @direction.y / ll
		@position.y = y - h * @direction.x / ll

		@font.id = id
	end

	def center_right=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Array
			x = pos[0][0]
			y = pos[0][1]
		end

		id = @font.id
		@font.id =  @font_id	if @font_id != nil
		l = @font.stringSize( @text ) * @scale
		h = @font.height * 0.5 * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x - l * @direction.x / ll + h * @direction.y / ll
		@position.y = y - l * @direction.y / ll - h * @direction.x / ll

		@font.id = id
	end

	def bottom_right=( *pos )
		if pos[0].kind_of? Point
			x = pos[0].x
			y = pos[0].y
		elsif pos[0].kind_of? Numeric
			x = pos[0]
			y = pos[1]
		elsif pos[0].kind_of? Array
			if pos[0][0].kind_of? Numeric
				x = pos[0][0]
				y = pos[0][1]
			end
		end

		id = @font.id
		@font.id = @font_id	if @font_id != nil
		l = @font.stringSize( @text ) * @scale
		ll = sqrt( @direction.x * @direction.x + @direction.y * @direction.y )
		@position.x = x - l * @direction.x / ll
		@position.y = y - l * @direction.y / ll

		@font.id = id
	end
end

# ̑傫̃NX
class JwFont
	# CONSTRUCTOR
	def initialize
		@id = 1
		@widthArray = Array.new
		@heightArray = Array.new
		@gapArray = Array.new
		@free_size = 2.0
		@scale = nil
	end

	attr	:id,				true
	attr	:scale,			true
	attr	:free_size,	true

	def width
		if @scale == nil
			scale = 1.0
		else
			scale = @scale.value
		end
		if @id == 0
			width = @free_size * scale
		else
			width = @widthArray[ @id ] * scale
		end
	end

	def height
		if @scale == nil
			scale = 1.0
		else
			scale = @scale.value
		end

		if @id == 0
			height = @free_size * scale
		else
			height = @heightArray[ @id ] * scale
		end
	end

	def gap
		if @scale == nil
			scale = 1.0
		else
			scale = @scale.value
		end

		if @id == 0
			gap = 0
		else
			gap = @gapArray[ @id ] * scale
		end
	end

	def set( str )
		case str
			when /^hcw/
				@widthArray = str.split
				@widthArray.collect!{ |item| item.to_f }

			when /^hch/
				@heightArray = str.split
				@heightArray.collect!{ |item| item.to_f }

			when /^hcd/
				@gapArray = str.split
				@gapArray.collect!{ |item| item.to_f }
		end
	end

	def setNearSize( font_size )
		@free_size = font_size
		found = false
		over = false
		for idx in 1 .. 10
			if @heightArray[ idx ] == font_size
				found = true
				break
			end

			if @heightArray[ idx ] > font_size
				over = true
				idx -= 1
				break
			end
		end

		if found or over
			@id = idx
		else
			@id = 10
		end
	end

	def draw( *out )
		if out[ 0 ].is_a? File
			if @id > 0
				out[ 0 ].printf "cn%d\n", @id
			else
				out[ 0 ].printf "cn0 %f %f 0 1\n", @free_size, @free_size
			end
		else
			if @id > 0
				printf "cn%d\n", @id
			else
				printf "cn0 %f %f 0 1\n", @free_size, @free_size
			end
		end
	end

	def stringSize( str )
		dummy_str = str.gsub( /(\^[\/ud\!\#]|\^\$[1-9])/, '' )
		size = self.width * 0.5 * dummy_str.length.to_f
		size += self.gap * (dummy_str.length - 1).to_f * 0.5
	end
end

# kڂ̃NX
class JwScale
	def initialize
		@scaleArray = []
		@layerGroup = 0
	end
	attr( "layerGroup", true )

	def set( jw_string )
		case jw_string
			when /^hs/
				@scaleArray = jw_string.split
				@scaleArray.collect!{ |item| item.to_f }
				@scaleArray.shift

			when /^lg(.*)/
				@layerGroup = $1.hex
		end
	end

	def value
		return @scaleArray[ @layerGroup ]
	end
end

# }̃NX
class JwPen
	attr	:color,		true
	attr	:type,		true

	def initialize
		@color = 1
		@type = 1
	end

	def set( jw_string )
		case jw_string
			when /^lc(.*)/
				@color = $1.strip.to_i

			when /^lt(.*)/
				@type = $1.strip.to_i
		end
	end

	def draw( *out )
		if out[ 0 ].is_a? File
			out[ 0 ].printf "lc%d\n", @color
			out[ 0 ].printf "lt%d\n", @type
		else
			printf "lc%d\n", @color
			printf "lt%d\n", @type
		end
	end
end

# CLASS: ݒNX
class NumberFormat
	attr	:fieldWidth
	attr	:precision
	attr	:number, true
	attr	:prefix, true
	attr	:suffix, true

	def initialize( width, prec )
		set width, prec
		@is_trancate = false
		@is_round5 = false
		@number = 0.0
	end

	def set( width, prec )
		if @is_tracate
			@fieldWidth = width + 1
			@precision = prec + 1
		else
			@fieldWidth = width
			@precision = prec
		end
		@form = sprintf( "%%%d.%df", @fieldWidth, @precision )
	end

	def trancated= ( sw )
		if @is_trancate != sw
			@is_trancate = sw

			if sw
				set( @fieldWidth + 1, @precision + 1 )
				@is_round5 = false
			else
				set( @fieldWidth - 1, @precision - 1 )
			end
		end
	end

	def trancated?
		return @is_trancate
	end

	def round5= ( sw )
		if @is_round5 != sw
			@is_round5 = sw

			if @is_trancate  and sw
				set( @fieldWidth - 1, @precision - 1 )
				@is_trancate = false
			end
		end
	end

	def round5?
		return @is_round5
	end

	def to_s
		if @is_round5
			 str = sprintf( @form, @number * 2 )
			 str.chop!
			 str = sprintf( @form, str.to_f * 0.5 )
		else
			str = sprintf( @form, @number )
		end

		if @is_trancate
			pform = Regexp.new '(\.' + '\d' * ( @precision - 1 ) + ')'
			pstr = @number.to_s
			pstr.scan( pform )
			str.sub!( pform, $1 ) if $1 != nil
			str.chop!
		end
		@number = str.to_f

		if @prefix != nil
			str = @prefix + str.strip
		end
		if @suffix != nil
			str += @suffix
		end

		str
	end

	# METHOD: IɊۂ߂
	def number!( new_number )
		@number = new_number
		to_s
		@number
	end

	# METHOD: GNZp
	def excel
		if @is_trancate
			form = "0." + ( "0" * ( @precision - 1 ) ) + '"  "'
		else
			form = "0." + ( "0" * @precision ) + '"  "'
		end
		return form
	end
end
