# JwUI.rb  revision 0.2
# JW-CAD + VisualuRuby UIi`

vr_DIR="vr/" unless vr_DIR
require vr_DIR+'vruby'
require vr_DIR+'vrcontrol' 
require vr_DIR+'vrcomctl'

# CݒR{{bNX
class LayerCombo < VRCombobox
	Layers = [ "()", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F" ]

	def vrinit
		super
		setListStrings Layers
	end

	def select( layer )
		if layer.kind_of? String
			if layer == "Z"
				super 0
			else
				super( layer.hex + 1 )
			end
		else
			super layer
		end
	end

	def selected
		layer = getTextOf( selectedString )
		if layer == "()"
			layer = "Z"
		end

		layer
	end
end

# FݒR{{bNX
class ColorCombo < VRCombobox
	LineColors = [ "()", "1", "2", "3", "4", "5", "6", "7", "8", "⏕" ]

	def vrinit
		super
		setListStrings LineColors
	end

	def select( color )
		if color.kind_of? String
			super color.to_i
		else
			super color
		end
	end

	def selected
		color = getTextOf( selectedString )
		case color
			when "()"
				color = "0"
			when "⏕"
				color = "9"
		end

		color
	end
end

# ݒR{{bNX
class LineCombo < VRCombobox
	LineTypes = [ "()", "", "_(1)", "_(2)", "_(3)", "_(1)", "_(2)", "_(1)", "_(2)", "⏕" ]

	def vrinit
		super
		setListStrings LineTypes
	end

	def select( style )
		if style.kind_of? String
			super style.to_i
		else
			super style
		end
	end

	def selected
		style = selectedString.to_s
	end
end

# _ݒR{{bNX
class PointCombo < VRCombobox
	PointTypes = [ "()", "1", "2", "3", "4", "5", "6", "7", "8", "⏕_" ]

	def vrinit
		super
		setListStrings PointTypes
	end

	def select( color )
		if color.kind_of? String
			case color
				when "C"
					super 10
				else
					super color.to_i
			end
		else
			super color
		end
	end

	def selected
		color = getTextOf( selectedString )
		case color
			when "()"
				color = "0"
			when "⏕_"
				color = "9"
			when "(~)"
				color = "C"
		end

		color
	end
end

# _ݒpl
class PointPanel < VRPanel
  include VRComCtlContainer
  include VRStdControlContainer

	attr	:pointColor
	attr	:radius
	attr	:circleColor

	def construct
		x = 0
		if self.caption != ''
			x = self.caption.length * 8 + 8
			addControl(VRStatic,		'static1',	self.caption,		0, 2, x, 24)
		end
		addControl(PointCombo,		'selectPoint',	'',					x, 0, 80, 220)
		@selectPoint.addString '(~)'
		addControl(VRStatic,			'static2',	'iaF',			x + 88, 2, 64, 24)
		addControl(VREdit,				'editRadius',	'1.0',				x + 152, 0, 64, 24)
		addControl(VRStatic,			'static3',	'mm@FF',			x + 216, 2, 64, 24)
		addControl(ColorCombo,		'selectColor',	'',					x + 280, 0, 80, 220)
		addControl(VRStatic,			'static4',	'j',						x + 360, 2, 16, 24)
	end

	def enabled=( sw )
		if sw
			@selectPoint.enabled = sw
			selectPoint_selchanged
		else
			@selectPoint.enabled = sw
			@editRadius.enabled = sw
			@selectColor.enabled = sw
		end
	end

	def radius=( r )
		@radius = r
		@editRadius.caption = @radius.to_s
	end

	def pointColor=( color )
		@selectPoint.select color
		selectPoint_selchanged
	end

	def circleColor=( color )
		@selectColor.select color
		@circleColor = color
	end

	def selectPoint_selchanged
		@pointColor = @selectPoint.selected
		if @pointColor == "C"
			@editRadius.enabled = true
			@selectColor.enabled = true
		else
			@editRadius.enabled = false
			@selectColor.enabled = false
		end
		call_parenthandler 'changed'
	end

	def editRadius_changed()
		@radius = @editRadius.caption.to_f
		call_parenthandler 'changed'
	end

	def selectColor_selchanged()
		@circleColor = @selectColor.selected
		call_parenthandler 'changed'
	end
end

# TCYݒR{{bNX
class FontCombo < VRCombobox
	FontSizes = [ "()", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" ]

	def vrinit
		super
		setListStrings FontSizes
	end

	def select( font )
		if font.kind_of? String
			super font.to_i
		else
			super font
		end
	end

	def selected
		font = getTextOf( selectedString )
		if font == "()"
			font = "0"
		end

		font
	end
end

# GNZt@C^V[gwpl
class ExcelSelector < VRPanel
	require 'win32ole'
  include VRComCtlContainer
  include VRStdControlContainer

	ExcelFilter = ["Excelt@C(*.xls)","*.xls"]
	AllFilter = ["SẴt@C(*.*)","*.*"]

	# PROPERTIES
	attr	:app,				true
	attr	:autoClose,	true
	attr	:saveAs,		true
	attr	:history,		true
	attr	:workbook
	attr	:file
	attr	:sheetName

	# CONSTRUCTOR
	def construct
		@app = WIN32OLE.new('Excel.Application')
		@workbook = nil
		@autoClose = true
		@saveAs = 'file_history'
		@history = 10
		@filters = [ ExcelFilter, AllFilter ]

		addControl(VRStatic,			'static1',		't@CF',		0,4,80,24)
		addControl(VREditCombobox,'editExcel',	'',							80,2,self.w - 176,122)
		addControl(VRButton,			'buttonExcel','Q...',			self.w - 80,0,80,28)

		addControl(VRStatic,			'static2',		'V[g@F',		0,34,80,24)
		addControl(VRListbox,			'listSheet',	'listSheet',		80,32,self.w - 176,self.h - 32)
	end

	# METHOD: addString
	#					t@CXgɍڂǉ
	def addString( str )
		@editExcel.addString str
	end

	def addFileType( *arr )
		idx = @filters.length - 1
		if arr[0].kind_of? Array
			@filters.push AllFilter
			@filters[ idx ] = arr[0]
		elsif arr[0].kind_of? String and arr.length == 2
			@filters.push AllFilter
			@filters[ idx ] = [ arr[0], arr[1] ]
		end
	end

	# METHOD:	save
	#					t@CXg̍ڂۑ
	def save( fileID )
		file_pathes = Array.new					# GNZt@C
		file_pathes.push @file
		@editExcel.eachString { |excel_file| file_pathes.push( excel_file ) }
		file_pathes.uniq!

		if file_pathes.length >= @history
			file_pathes = file_pathes[ 0..(@history - 1) ]
		end
		idx = 0
		for file_path in file_pathes
			fileID.printf "%s%d = %s\n", @saveAs, idx, file_path
			idx += 1
		end
	end

	# Loacal Event Handlers
	def buttonExcel_clicked
		selected = SWin::CommonDialog::openFilename(
																		self,
																		@filters
		)
    if selected
			@editExcel.caption = selected
			@file = selected

			if FileTest.exist? @file
				if @file =~ /\.(xls|XLS)$/
					@listSheet.enabled = true
					updateSheetList
				else
					@listSheet.clearStrings
					@listSheet.enabled = false
				end
				call_parenthandler 'changed'
			end
		end
	end

	def editExcel_selchanged
		@file = @editExcel.getTextOf( @editExcel.selectedString )
		if FileTest.exist? @file
			if @file =~ /\.(xls|XLS)$/
				@listSheet.enabled = true
				updateSheetList
			else
				@listSheet.clearStrings
				@listSheet.enabled = false
			end
			call_parenthandler 'changed'
		end
	end

	def editExcel_changed
		@file = @editExcel.caption
		if FileTest.exist? @file
			if @file =~ /\.(xls|XLS)$/
				@listSheet.enabled = true
				updateSheetList
			else
				@listSheet.clearStrings
				@listSheet.enabled = false
			end
			call_parenthandler 'changed'
		end
	end

	def listSheet_selchanged
		@sheetName = @listSheet.getTextOf( @listSheet.selectedString )
		call_parenthandler 'sheetchanged'
	end

	private
	def updateSheetList()
		if @workbook !=nil
			@workbook.close false
		end
		@workbook = @app.workbooks.open @file, 0, true
		@listSheet.clearStrings
		activeIndex = 0
		if @workbook.activeSheet != nil
			activeName = @workbook.activeSheet.name
		end 
		for idx in 1 .. @workbook.worksheets.count
			sheetName = @workbook.worksheets( idx ).name
			@listSheet.addString idx - 1, sheetName
			activeIndex =  idx - 1	if activeName == sheetName
		end
		@listSheet.select activeIndex
		listSheet_selchanged
		if @autoClose
			@workbook.close false
			@workbook = nil
		end
	end
end

# t@Cwpl
class FileSelector < VRPanel
  include VRComCtlContainer
  include VRStdControlContainer

	TextFilter = ["eLXgt@C(*.txt)","*.txt"]
	AllFilter = ["SẴt@C(*.*)","*.*"]

	# PROPERTIES
	attr	:app,				true
	attr	:saveAs,		true
	attr	:history,		true
	attr	:file

	# CONSTRUCTOR
	def construct
		@saveAs = 'file_history'
		@history = 10
		@filters = [ TextFilter, AllFilter ]

		x = self.caption.length * 8
		addControl(VRStatic,			'static1',		self.caption,	0,4,x,24)
		addControl(VREditCombobox,'editFile',	'',							x + 8,2,self.w - 96 - x,122)
		addControl(VRButton,			'buttonFile','Q...',			self.w - 80,0,80,28)
	end

	# METHOD: addString
	#					t@CXgɍڂǉ
	def addString( str )
		@editFile.addString str
	end

	def addFileType( *arr )
		idx = @filters.length - 1
		if arr[0].kind_of? Array
			@filters.push AllFilter
			@filters[ idx ] = arr[0]
		elsif arr[0].kind_of? String and arr.length == 2
			@filters.push AllFilter
			@filters[ idx ] = [ arr[0], arr[1] ]
		end
	end

	# METHOD:	save
	#					t@CXg̍ڂۑ
	def save( fileID )
		file_pathes = Array.new					# t@C
		file_pathes.push @file
		@editFile.eachString { |file| file_pathes.push( file ) }
		file_pathes.uniq!

		if file_pathes.length >= @history
			file_pathes = file_pathes[ 0..(@history - 1) ]
		end
		idx = 0
		for file_path in file_pathes
			fileID.printf "%s%d = %s\n", @saveAs, idx, file_path
			idx += 1
		end
	end

	# Loacal Event Handlers
	def buttonFile_clicked
		selected = SWin::CommonDialog::openFilename(
																		self,
																		@filters
		)
    if selected
			@editFile.caption = selected
			@file = selected
			call_parenthandler 'changed'
		end
	end

	def editFile_selchanged
		@file = @editFile.getTextOf( @editFile.selectedString )
		call_parenthandler 'changed'
	end

	def editExcel_changed
		@file = @editFile.caption
		call_parenthandler 'changed'
	end
end
